/* Project name:
     CF_Write (Demonstration of MMC Library usage)
 * Copyright:
     (c) MikroElektronika, 2010.
 * Revision History
     2006:
     - Initial release;
     20101130(TL):
     - Adapted for PRO compilers
 * Description:
     This project demonstrates usage of the Compact Flash Library routines,
     accessing and writing to the CF (Compact Flash Card) storage media.
     Upon initialization, this example stores temperature sensor readings
     in the CF card at sector 590
 * Test configuration:

     MCU:             ATmega128
                      http://www.atmel.com/dyn/resources/prod_documents/doc2467.pdf
     Dev.Board:       BIGAVR6
                      http://www.mikroe.com/eng/products/view/322/bigavr6-development-system/
     Oscillator:      HS, 8.000 MHz
     Ext. Modules:    ac:Compact_Flash_Board
                      http://www.mikroe.com/eng/products/view/131/compact-flash-board/
                      ac:DS1820
                      http://www.mikroe.com/eng/products/view/277/various-components/
     SW:              mikroC PRO for AVR
                      http://www.mikroe.com/eng/products/view/228/mikroc-pro-for-avr/

 * NOTES:
     - PORTB (CF ctrl), PORD (CF Data), LEDs = OFF
     - PORTC LEDs switch = ON (for displaying data)
     - Temperature sensor DS1820 needs to be connected on RB0 (SW14.1 - ON)
*/

#include "built_in.h"

// set compact flash pinout
char Cf_Data_Port at PORTD;
char Cf_Data_Port_Direction at DDRD;

sbit CF_RDY at PINB7_bit;
sbit CF_WE  at PORTB6_bit;
sbit CF_OE  at PORTB5_bit;
sbit CF_CD1 at PINB4_bit;
sbit CF_CE1 at PORTB3_bit;
sbit CF_A2  at PORTB2_bit;
sbit CF_A1  at PORTB1_bit;
sbit CF_A0  at PORTB0_bit;

sbit CF_RDY_direction at DDB7_bit;
sbit CF_WE_direction  at DDB6_bit;
sbit CF_OE_direction  at DDB5_bit;
sbit CF_CD1_direction at DDB4_bit;
sbit CF_CE1_direction at DDB3_bit;
sbit CF_A2_direction  at DDB2_bit;
sbit CF_A1_direction  at DDB1_bit;
sbit CF_A0_direction  at DDB0_bit;
// end of cf pinout

unsigned i, temp;
char j;
// Set TEMP_RESOLUTION to the corresponding resolution of your DS18x20 sensor:
//  18S20: 9  (default setting; can be 9,10,11,or 12)
//  18B20: 12
const unsigned short TEMP_RESOLUTION = 9;

void Write_Temperature(unsigned int temp) {
  const unsigned short RES_SHIFT = TEMP_RESOLUTION - 8;
  char *text;
  char temp_whole;
  unsigned int temp_fraction;
  char k;

  text = "000.0000";
  if (temp & 0x8000) {
     text[0] = '-';
     temp = ~temp + 1;
     }

  temp_whole = temp >> RES_SHIFT ;
  if (temp_whole/100)
     text[0] = temp_whole/100     + 48;

  text[1] = (temp_whole/10)%10 + 48;
  text[2] = temp_whole%10      + 48;

  temp_fraction = temp << (4-RES_SHIFT);
  temp_fraction &= 0x000F;
  temp_fraction *= 625;
  text[4] = temp_fraction/1000     + 48;
  text[5] = (temp_fraction/100)%10 + 48;
  text[6] = (temp_fraction/10)%10  + 48;
  text[7] = temp_fraction%10       + 48;

  for(k=0 ; k<8 ; k++)
     Cf_Write_Byte(text[k]);
}

void main() {
  //--- PORTE is input
  PORTE  = 0xFF;
  DDRE  = 0x0F;

  PORTC = 0xFF;               // For monitoring progress
  DDRC = 0;
  Delay_ms(1000);

  Cf_Init();                  // Initialize CF ports
  while (!Cf_Detect()) ;      // Loop until CF card is initialized

  PORTC = 0x0F;
  Delay_ms(1000);

  Cf_Write_Init(590, 1);      // Initialize CF write at sector address 590

  //--- main loop
  for(i=0 ; i<64 ; i++) {     // 64 x 8bytes = 512bytes
    PORTC = i;

    Ow_Reset(&PORTB, 0);            // Onewire reset signal
    Ow_Write(&PORTB, 0, 0xCC);      // Issue command SKIP_ROM
    Ow_Write(&PORTB, 0, 0x44);      // Issue command CONVERT_T
    Delay_us(120);

    Ow_Reset(&PORTB, 0);
    Ow_Write(&PORTB, 0, 0xCC);      // Issue command SKIP_ROM
    Ow_Write(&PORTB, 0, 0xBE);      // Issue command READ_SCRATCHPAD
    Delay_ms(400);

    j    = OW_Read(&PORTB, 0);      // Get temperature LSB
    temp = OW_Read(&PORTB, 0);      // Get temperature MSB
    temp <<= 8;
    temp += j;                      // Form the 2byte variable
    Write_Temperature(temp);

    Delay_ms(100);
  }

}