/* Project name:
     CF_Write (Demonstration of MMC Library usage)
 * Copyright:
     (c) MikroElektronika, 2010.
 * Revision History
     2006:
     - Initial release;
     20101130(TL):
     - Adapted for PRO compilers
 * Description:
     This project demonstrates usage of the Compact Flash Library routines,
     accessing and writing to the CF (Compact Flash Card) storage media.
     Upon initialization, this example stores temperature sensor readings
     in the CF card at sector 590
 * Test configuration:
     
     MCU:             PIC18F4520
                      http://ww1.microchip.com/downloads/en/devicedoc/39564c.pdf
     Dev.Board:       EasyPIC6
                      http://www.mikroe.com/eng/products/view/297/easypic6-development-system/
     Oscillator:      HS-PLL 4x, 32.00000 MHz
     Ext. Modules:    ac:Compact_Flash_Board
                      http://www.mikroe.com/eng/products/view/131/compact-flash-board/
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/eng/products/view/9/mikroc-pro-for-pic/

 * NOTES:
     - PORTB (CF ctrl), PORTD (CF Data), LEDS = OFF
     - PORTC LEDs switch = ON (for displaying data)
*/

#include "built_in.h"

// set compact flash pinout
char Cf_Data_PORT at PORTD;

sbit CF_RDY at RB7_bit;
sbit CF_WE  at LATB6_bit;  // for writing to output pin always use latch (PIC18 family)
sbit CF_OE  at LATB5_bit;  // for writing to output pin always use latch (PIC18 family)
sbit CF_CD1 at RB4_bit;
sbit CF_CE1 at LATB3_bit;  // for writing to output pin always use latch (PIC18 family)
sbit CF_A2  at LATB2_bit;  // for writing to output pin always use latch (PIC18 family)
sbit CF_A1  at LATB1_bit;  // for writing to output pin always use latch (PIC18 family)
sbit CF_A0  at LATB0_bit;  // for writing to output pin always use latch (PIC18 family)

sbit CF_RDY_direction at TRISB7_bit;
sbit CF_WE_direction  at TRISB6_bit;
sbit CF_OE_direction  at TRISB5_bit;
sbit CF_CD1_direction at TRISB4_bit;
sbit CF_CE1_direction at TRISB3_bit;
sbit CF_A2_direction  at TRISB2_bit;
sbit CF_A1_direction  at TRISB1_bit;
sbit CF_A0_direction  at TRISB0_bit;
// end of cf pinout

unsigned i, temp;
char j;
// Set TEMP_RESOLUTION to the corresponding resolution of your DS18x20 sensor:
//  18S20: 9  (default setting; can be 9,10,11,or 12)
//  18B20: 12
const unsigned short TEMP_RESOLUTION = 9;

void Write_Temperature(unsigned int temp) {
  const unsigned short RES_SHIFT = TEMP_RESOLUTION - 8;
  char *text;
  char temp_whole;
  unsigned int temp_fraction;
  char k;

  text = "000.0000";
  if (temp & 0x8000) {
     text[0] = '-';
     temp = ~temp + 1;
     }

  temp_whole = temp >> RES_SHIFT ;
  if (temp_whole/100)
     text[0] = temp_whole/100     + 48;

  text[1] = (temp_whole/10)%10 + 48;
  text[2] = temp_whole%10      + 48;

  temp_fraction = temp << (4-RES_SHIFT);
  temp_fraction &= 0x000F;
  temp_fraction *= 625;
  text[4] = temp_fraction/1000     + 48;
  text[5] = (temp_fraction/100)%10 + 48;
  text[6] = (temp_fraction/10)%10  + 48;
  text[7] = temp_fraction%10       + 48;

  for(k=0 ; k<8 ; k++)
     Cf_Write_Byte(text[k]);
}

void main() {
  //--- Configure RE2 pin as digital I/O
  ADCON1 = 0xFF;
  //--- PORTE is input
  PORTE  = 0xFF;
  TRISE  = 0x0F;

  PORTC = 0xFF;               // For monitoring progress
  TRISC = 0;
  Delay_ms(1000);

  Cf_Init();                  // Initialize CF PORTs
  while (!Cf_Detect()) ;      // Loop until CF card is initialized

  PORTC = 0x0F;
  Delay_ms(1000);

  Cf_Write_Init(590, 1);      // Initialize CF write at sector address 590

  //--- main loop
  for(i=0 ; i<64 ; i++) {     // 64 x 8bytes = 512bytes
    PORTC = i;

    Ow_Reset(&PORTE,2);            // Onewire reset signal
    Ow_Write(&PORTE,2,0xCC);       // Issue command SKIP_ROM
    Ow_Write(&PORTE,2,0x44);       // Issue command CONVERT_T
    Delay_us(120);

    Ow_Reset(&PORTE,2);
    Ow_Write(&PORTE,2,0xCC);       // Issue command SKIP_ROM
    Ow_Write(&PORTE,2,0xBE);       // Issue command READ_SCRATCHPAD
    Delay_ms(400);

    j    = OW_Read(&PORTE,2);      // Get temperature LSB
    temp = OW_Read(&PORTE,2);      // Get temperature MSB
    temp <<= 8;
    temp += j;                     // Form the 2byte variable
    Write_Temperature(temp);

    Delay_ms(100);
  }

}