/* Project name:
     Cf_FAT_Read (Demonstration on usage of Cf_Fat16 library)
 * Copyright:
     (c) MikroElektronika 2010
 * Revision History
     2006:
     - Initial release;
     20101130(TL):
     - Adapted for PRO compilers;
 * Description:
     This project consists of several blocks that demonstrate various aspects of
     usage of the Cf_Fat16 library. These are:
     - Opening existing file and appending data to it (writing from end-of-file);
     - Opening a file and reading data from it (sending it to USART terminal);
     - Reading file contents;
 * Test configuration:
       
     MCU:             dsPIC30F6014A
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70566A.pdf
     Dev.Board:       BIGdsPIC6
                      http://www.mikroe.com/eng/products/view/483/bigdspic6-development-system/
     Oscillator:      HS-PLL 8x, 80.000 MHz
     Ext. Modules:    ac:Compact_Flash_Board
                      http://www.mikroe.com/eng/products/view/131/compact-flash-board/
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/eng/products/view/231/mikroc-pro-for-dspic30-33-and-pic24/
         
 * NOTES:
     - PORTD(CF ctrl), PORTF(CF Data), LEDs = OFF
     - RS232 jumpers set to Rx = RF2 and Tx = RF3
     - Please make sure that CF card is properly formatted (to FAT16 or just FAT)
       before testing it on this example!
     - This example expects CF card to be inserted before reset, otherwise,
       the FAT_ERROR message is displayed!!!
*/

#include "built_in.h"

// set compact flash pinout
char Cf_Data_Port at PORTF;

sbit CF_RDY at RD7_bit;
sbit CF_WE  at RD6_bit;
sbit CF_OE  at RD5_bit;
sbit CF_CD1 at RD4_bit;
sbit CF_CE1 at RD3_bit;
sbit CF_A2  at RD2_bit;
sbit CF_A1  at RD1_bit;
sbit CF_A0  at RD0_bit;

sbit CF_RDY_direction at TRISD7_bit;
sbit CF_WE_direction  at TRISD6_bit;
sbit CF_OE_direction  at TRISD5_bit;
sbit CF_CD1_direction at TRISD4_bit;
sbit CF_CE1_direction at TRISD3_bit;
sbit CF_A2_direction  at TRISD2_bit;
sbit CF_A1_direction  at TRISD1_bit;
sbit CF_A0_direction  at TRISD0_bit;
// end of cf pinout

char
 FAT_ERROR[20] = "FAT16 not found",
 file_contents[50] = "XX Compact Flash FAT16 library by Anton Rieckert\n";

char
 filename[14] = "MIKRO00xTXT";          // File names
unsigned short
 tmp, caracter, loop, loop2;
unsigned long
 i, size;

//-------------- Writes string to USART
void I_Write_Str(char *ostr) {
  unsigned short i;

  i = 0;
  while (ostr[i]) {
    UART1_Write(ostr[i++]);
  }
  UART1_Write(0x0A);
}

//-------------- Creates new file and writes some data to it
    void M_Create_New_File() {
  filename[7] = 'A';
  Cf_Fat_Assign(&filename, 1);     // Will not find file and then create file
  Cf_Fat_Rewrite();                // To clear file and start with new data
  for(loop = 1; loop <= 99; loop++) { //  We want 5 files on the MMC card
    file_contents[0] = loop / 10 + 48;
    file_contents[1] = loop % 10 + 48;
    Cf_Fat_Write(file_contents, 42); // write data to the assigned file
  }
}

//-------------- Opens an existing file and rewrites it
void M_Open_File_Rewrite() {

  filename[7] = 'G';
  Cf_Fat_Assign(&filename, 0);
  Cf_Fat_Rewrite();
  for(loop = 1; loop <= 55; loop++) {
    file_contents[0] = loop / 10 + 64;
    file_contents[1] = loop % 10 + 64;
    Cf_Fat_Write(file_contents, 42); // write data to the assigned file
  }
}

//-------------- Opens an existing file and appends data to it
//               (and alters the date/time stamp)
void M_Open_File_Append() {

     filename[7] = 'F';
     Cf_Fat_Assign(&filename, 0);
     Cf_Fat_Set_File_Date(2005,6,21,10,35,0);
     Cf_Fat_Append();                                    // Prepare file for append
     Cf_Fat_Write(" for mikroElektronika 2005\n", 27);   // Write data to assigned file
}

//-------------- Deletes a file. If file doesn't exist, it will first be created
//               and then deleted.
void M_Delete_File() {
  filename[7] = 'B';
  Cf_Fat_Assign(filename, 0);
  Cf_Fat_Delete();
}

//-------------- Opens an existing file, reads data from it and puts it to USART
void M_Open_File_Read() {

  filename[7] = 'F';
  Cf_Fat_Assign(&filename, 0);
  Cf_Fat_Reset(&size);            // To read file, procedure returns size of file
  for (i = 1; i <= size; i++) {
    Cf_Fat_Read(&caracter);
    UART1_Write(caracter);        // Write data to USART
  }
}

//-------------- Creates many new files and writes data to them
void M_Create_Multiple_Files() {
  for(loop2 = 'A'; loop2 <= 'Z'; loop2++) {
    UART1_Write(loop2);
    filename[7] = loop2;             // set filename
    Cf_Fat_Assign(&filename, 1);     // find existing file or create a new one
    Cf_Fat_Rewrite();                // To clear file and start with new data
    for(loop = 1; loop <= 44; loop++) {
      file_contents[0] = loop / 10 + 48;
      file_contents[1] = loop % 10 + 48;
      Cf_Fat_Write(file_contents, 42); // write data to the assigned file
    }
  }
}

//-------------- Tests whether file exists, and if so sends its creation date
//               and file size via USART
void M_Test_File_Exist() {
  unsigned long fsize;
  unsigned int year;
  unsigned short month, day, hour, minute;
  unsigned char outstr[12];

  filename[7] = 'F';         //uncomment this line to search for file that DOES exists
//  filename[7] = 'B';       //uncomment this line to search for file that DOES NOT exist
  if (Cf_Fat_Assign(filename, 0)) {
    //--- file has been found - get its date
    Cf_Fat_Get_File_Date(&year, &month, &day, &hour, &minute);
    WordToStr(year, outstr);
    I_Write_Str(outstr);
    ByteToStr(month, outstr);
    I_Write_Str(outstr);
    WordToStr(day, outstr);
    I_Write_Str(outstr);
    WordToStr(hour, outstr);
    I_Write_Str(outstr);
    WordToStr(minute, outstr);
    I_Write_Str(outstr);
    //--- get file size
    fsize = Cf_Fat_Get_File_Size();
    LongToStr((signed long)fsize, outstr);
    I_Write_Str(outstr);
  }
  else {
    //--- file was not found - signal it
    UART1_Write(0x55);
    Delay_ms(1000);
    UART1_Write(0x55);
  }
}

//-------------- Main. Uncomment the function(s) to test the desired operation(s)
void main() {
     UART1_Init(9600);            // Set up USART for the read of the files
     Delay_ms(100);
     tmp = Cf_Fat_Init();
     if(!tmp || (tmp == 255))     // Init the FAT library
       I_Write_Str(FAT_ERROR);
     else {
       //--- Test start
       M_Create_New_File();
       M_Open_File_Append();
       I_Write_Str("Reading file");
       M_Open_File_Read();
       M_Test_File_Exist();
         //--- Test termination
       I_Write_Str("End");
     }
}